// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#include <sourcemod>
#include <sdktools>
#include <cstrike>

#pragma semicolon 1

#define PLUGIN_VERSION            "1.58"
#define PLUGIN_VERSION_CVAR       "forlix_soscore_version"

#define PLUGIN_COMMAND            "soscore"
#define PLUGIN_COMMAND_DEADTIME   0.75

#define PANEL_DISPLAYTIME         75

#define ANNOUNCEMENT_PREFIX       "\x04Forlix SoScore\x01 -"

#define OWN_GOAL_BAN_REASON       "Temporary ban for %s (%u own goals)"
#define OWN_GOAL_BAN_REASON_PERM  "Permanent ban (%u own goals)"

#define NOT_SOCCER_MAP_MSG        "Forlix SoScore unloads (not a known soccer map)"
#define ERROR_MSG_FATAL           "Fatal error occurred (detailed error message(s) in SourceMod logs)"

#define LOG_MSG_GOAL              "\"%N<%u><%s><%s>\" scored a goal"
#define LOG_MSG_OWNGOAL           "\"%N<%u><%s><%s>\" scored own goal"
#define LOG_MSG_ASSIST            "\"%N<%u><%s><%s>\" assisted"

#define MAX_MISSTRIGS_PER_TEAM    8
#define MAX_KEEPER_NUM            16
#define MAX_MSG_LEN               128

#define TRANSLATION_FILE          "forlix_soscore.phrases.txt"
#define CONFIG_PATH               "configs/forlix_soscore.cfg"

#define REFERENCE_FIELD_NAME      "ka_soccer_2006_v3fix2"
#define REFERENCE_FIELD_SIZE      2855.0

#define IN_FRONT_OF_GOAL_RADIUS   250.0
#define MIN_SAVE_DISTANCE         250.0
#define MIN_PASS_DISTANCE         500.0
#define MIN_AIR_PASS_HEIGHT       100.0
#define MIN_MISS_INTERVAL         2.0

public Plugin:myinfo =
{
  name = "Forlix SoScore",
  author = "Forlix (Dominik Friedrichs)",
  description = "Soccer scorekeeping and goal announcements",
  version = PLUGIN_VERSION,
  url = "http://forlix.org/"
};

new Handle:h_forward_nosoccermap = INVALID_HANDLE;
new Handle:h_forward_entitiesread = INVALID_HANDLE;
new Handle:h_forward_ballshot = INVALID_HANDLE;
new Handle:h_forward_ballpassed = INVALID_HANDLE;
new Handle:h_forward_ballturnedover = INVALID_HANDLE;
new Handle:h_forward_goalscored = INVALID_HANDLE;
new Handle:h_forward_goalmissed = INVALID_HANDLE;
new Handle:h_forward_goalsaved = INVALID_HANDLE;

new log_goal_scores = 0;
new announcement_type = 0;

new spawn_mode = 0;
new respawn_blocktime = 0;

new own_goal_ban_num = 0;
new own_goal_ban_time = 0;

new player_godmode_health = 0;
new team_limit = 0;

new String:help_cmd[32];

new sv_nomvp = 0;

new shooter_userid = 0;
new lastshooter_userid = 0;
new shooter_team = 0;
new lastshooter_team = 0;

new Float:ball_shottime = 0.0;
new Float:ball_shotorigin[3];
new bool:air_shot = false;

new Float:round_freeze_end = 0.0;

new bool:tracking_killed = false;
new bool:roundend = false;
new bool:round_abouttoend = false;
new bool:roundreset = false;
new bool:printctmsg = false;

new game_score_ref = INVALID_ENT_REFERENCE;
new soccerball_ref = INVALID_ENT_REFERENCE;
new goaltrig_t_ref = INVALID_ENT_REFERENCE;
new goaltrig_ct_ref = INVALID_ENT_REFERENCE;
new misstrig_t_refs[MAX_MISSTRIGS_PER_TEAM+1] = {INVALID_ENT_REFERENCE, ...};
new misstrig_ct_refs[MAX_MISSTRIGS_PER_TEAM+1] = {INVALID_ENT_REFERENCE, ...};

new String:centermsg_type[64];

new p_score[MAXPLAYERS+1];
new p_assist[MAXPLAYERS+1];
new p_owngoal[MAXPLAYERS+1];

new bool:p_classmenublocks[MAXPLAYERS+1];
new bool:p_choosingclass[MAXPLAYERS+1];
new p_teamchooselock[MAXPLAYERS+1];

new bool:p_spawned[MAXPLAYERS+1];
new bool:p_spawned_in_round[MAXPLAYERS+1];
new bool:p_died_from_goal[MAXPLAYERS+1];

new Float:field_size = 0.0;

new Float:vec_tgoal_origin[3];
new Float:vec_ctgoal_origin[3];

new Float:vec_tgoal_to_ctgoal[3];
new Float:vec_ctgoal_to_tgoal[3];

#include "fs_convars.sp"
#include "fs_entities.sp"
#include "fs_events.sp"
#include "fs_soevents.sp"
#include "fs_respawn.sp"
#include "fs_teamlimit.sp"
#include "fs_toolfuncs.sp"
#include "fs_forwards.sp"
