// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#include "_main.inc"

static bool:late_load = false;


public APLRes:AskPluginLoad2(Handle:myself,
                             bool:late,
                             String:error[],
                             err_max)
{
  late_load = late;
  return(APLRes_Success);
}


public OnPluginStart()
{
  RegPluginLibrary("forlix_soscore");

  HookEvent("round_start", Event_RoundStart);
  HookEvent("round_freeze_end", Event_RoundFreezeEnd);
  HookEvent("round_end", Event_RoundEnd);

  HookEvent("player_spawn", Event_PlayerSpawn);
  HookEvent("player_death", Event_PlayerDeath);
  HookEvent("player_team", Event_PlayerTeam);

  RegConsoleCmd(PLUGIN_COMMAND, HelpPanel, "Opens the SoScore Help panel.");

  RegConsoleCmd("jointeam", JoinTeam);
  RegConsoleCmd("joinclass", JoinClass);

  RegConsoleCmd("sprint", SprintDummy);

  HookUserMessage(GetUserMessageId("VGUIMenu"), UserMsg_VGUIMenu, false);
  HookUserMessage(GetUserMessageId("Damage"), UserMsg_Damage, true);

  CreateForwards();
  SetupConVars();

  return;
}


public OnPluginEnd()
{
  ResetPlayerHealth(true);
  return;
}


public OnMapStart()
{
  decl String:map[32];
  GetCurrentMap(map, sizeof(map));

  if(ReadMapConfig(map))
  // config read and it is a soccer map
  {
    LoadTranslations(TRANSLATION_FILE);

    if(late_load)
      LateLoad();
  }
  else
  {
    // ForlixSoScore_OnNoSoccerMap forward
    Call_StartForward(h_forward_nosoccermap);
    Call_Finish();

    UnloadWithMessage(NOT_SOCCER_MAP_MSG);
  }

  late_load = false;
  return;
}


public OnMapEnd()
{
  KillTriggers();
  return;
}


public OnGameFrame()
{
  static skip = 0;

  if(skip++ < 16
  || tracking_killed)
    return;

  decl Float:ball_noworigin[3];
  GetBallOrigin(ball_noworigin);

  if(ball_noworigin[2]-vec_tgoal_origin[2]
  >= MIN_AIR_PASS_HEIGHT)
    air_shot = true;

  skip = 0;
  return;
}


public OnClientConnected(client)
{
  p_score[client] = 0;
  p_assist[client] = 0;

  p_owngoal[client] = 0;

  p_spawned[client] = false;
  p_spawned_in_round[client] = false;
  p_died_from_goal[client] = false;

  p_classmenublocks[client] = false;
  p_choosingclass[client] = false;
  p_teamchooselock[client] = 0;

  return;
}


public OnClientDisconnect(client)
{
  AbortRespawnBlocktimer(client);
  return;
}


public Action:UserMsg_Damage(UserMsg:msg_id,
                             Handle:bf,
                             const players[],
                             playersnum,
                             bool:reliable,
                             bool:init)
{
  if(player_godmode_health)
  // block damage indicators when in godmode
    return(Plugin_Handled);

  return(Plugin_Continue);
}


public Action:SprintDummy(client, args)
{
  return(Plugin_Continue);
}


public Action:HelpPanel(client, args)
{
  if(!client)
    return(Plugin_Continue);

  if(DeadTimeBlock(client))
    return(Plugin_Handled);

  // respond to plugin command
  decl String:buf[256];

  new Handle:pn = CreatePanel();
  SetGlobalTransTarget(client);

  Format(buf, sizeof(buf), "Forlix SoScore v%s\n ", PLUGIN_VERSION);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "%t\n ", "HELPPANEL_TEXT");
  DrawPanelText(pn, buf);

  DrawPanelText(pn, "(c) 2008-2011 Forlix\nEmail: df@forlix.org\nWeb: http://forlix.org/\n ");

  Format(buf, sizeof(buf), "0. %t", "EXIT");
  DrawPanelText(pn, buf);

  SendPanelToClient(pn, client, HelpPanelClose, PANEL_DISPLAYTIME);
  CloseHandle(pn);

  return(Plugin_Handled);
}


public HelpPanelClose(Handle:panel,
                      MenuAction:action,
                      param1,
                      param2)
{
  return;
}


UnloadWithMessage(const String:message[])
{
  PrintToServer("%s", message);
  ServerCommand("sm plugins unload forlix_soscore");

  return;
}
