// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

static String:str_soccerball[32];
static String:str_goaltrig_t[32];
static String:str_goaltrig_ct[32];
static String:str_misstrig[32];

static String:str_soccerball_class[32];
static String:str_soccerball_outp[32];


bool:ReadMapConfig(const String:map[])
{
  decl String:path[PLATFORM_MAX_PATH];
  BuildPath(Path_SM, path, sizeof(path), CONFIG_PATH);

  new Handle:h_soccer_maps = CreateKeyValues("soccer_maps");

  if(!FileToKeyValues(h_soccer_maps, path))
    SetFailState("Configuration file \"%s\" not found", path);

  if(!KvJumpToKey(h_soccer_maps, map))
  // map not found in config file
  {
    CloseHandle(h_soccer_maps);
    return(false);
  }

  KvGetString(h_soccer_maps, "soccerball", str_soccerball, sizeof(str_soccerball));
  KvGetString(h_soccer_maps, "goaltrig_t", str_goaltrig_t, sizeof(str_goaltrig_t));
  KvGetString(h_soccer_maps, "goaltrig_ct", str_goaltrig_ct, sizeof(str_goaltrig_ct));
  KvGetString(h_soccer_maps, "misstrig", str_misstrig, sizeof(str_misstrig));

  new bool:errorflag = false;

  if(FindEntityByTargetname(-1, str_soccerball, "func_physbox") != -1)
  {
    strcopy(str_soccerball_class, sizeof(str_soccerball_class), "func_physbox");
    strcopy(str_soccerball_outp, sizeof(str_soccerball_outp), "OnDamaged");
  }
  else
  if(FindEntityByTargetname(-1, str_soccerball, "prop_physics") != -1)
  {
    strcopy(str_soccerball_class, sizeof(str_soccerball_class), "prop_physics");
    strcopy(str_soccerball_outp, sizeof(str_soccerball_outp), "OnHealthChanged");
  }
  else
  {
    LogError("Entity \"%s\" not found or it is neither \"func_physbox\" nor \"prop_physics\"",
    str_soccerball);
    errorflag = true;
  }

  if(FindEntityByTargetname(-1, str_goaltrig_t, "trigger_once") == -1)
  {
    LogError("Entity \"%s\" (class \"trigger_once\") not found",
    str_goaltrig_t);
    errorflag = true;
  }

  if(FindEntityByTargetname(-1, str_goaltrig_ct, "trigger_once") == -1)
  {
    LogError("Entity \"%s\" (class \"trigger_once\") not found",
    str_goaltrig_ct);
    errorflag = true;
  }

  if(FindEntityByTargetname(-1, str_misstrig, "trigger_multiple") == -1)
  {
    LogError("Entity \"%s\" (class \"trigger_multiple\") not found",
    str_misstrig);
    errorflag = true;
  }

  if(errorflag)
    SetFailState(ERROR_MSG_FATAL);

  CloseHandle(h_soccer_maps);
  return(true);
}


SetupEntities()
{
  // instantiate a game_score entity for score updates
  game_score_ref = EntIndexToEntRef(CreateEntityByName("game_score", -1));
  DispatchSpawn(game_score_ref);

  // rehook entity outputs and store references
  new soccerball_idx = FindEntityByTargetname(-1, str_soccerball, str_soccerball_class);

  new goaltrig_t_idx = FindEntityByTargetname(-1, str_goaltrig_t, "trigger_once");
  new goaltrig_ct_idx = FindEntityByTargetname(-1, str_goaltrig_ct, "trigger_once");

  if(soccerball_idx <= 0
  || goaltrig_t_idx <= 0
  || goaltrig_ct_idx <= 0)
  {
    LogError("Entity not found (BL %i, TG %i, CTG %i)",
             soccerball_idx, goaltrig_t_idx, goaltrig_ct_idx);

    SetFailState(ERROR_MSG_FATAL);
    return;
  }

  soccerball_ref = EntIndexToEntRef(soccerball_idx);
  goaltrig_t_ref = EntIndexToEntRef(goaltrig_t_idx);
  goaltrig_ct_ref = EntIndexToEntRef(goaltrig_ct_idx);

  GetEntPropVector(goaltrig_t_ref, Prop_Data, "m_vecAbsOrigin", vec_tgoal_origin);
  GetEntPropVector(goaltrig_ct_ref, Prop_Data, "m_vecAbsOrigin", vec_ctgoal_origin);

  // find all miss triggers
  // and detect which goal they belong to
  new index = -1;
  new t_mtnum = 0;
  new ct_mtnum = 0;

  while((index = FindEntityByTargetname(index, str_misstrig, "trigger_multiple")) > 0)
  {
    decl Float:origin[3];
    GetEntPropVector(index, Prop_Data, "m_vecAbsOrigin", origin);

    if(GetVectorDistance(origin, vec_tgoal_origin, true)
    <  GetVectorDistance(origin, vec_ctgoal_origin, true))
    {
      if(t_mtnum < MAX_MISSTRIGS_PER_TEAM)
        misstrig_t_refs[t_mtnum++] = EntIndexToEntRef(index);
    }
    else
    {
      if(ct_mtnum < MAX_MISSTRIGS_PER_TEAM)
        misstrig_ct_refs[ct_mtnum++] = EntIndexToEntRef(index);
    }
  }

  misstrig_t_refs[t_mtnum] = INVALID_ENT_REFERENCE;
  misstrig_ct_refs[ct_mtnum] = INVALID_ENT_REFERENCE;

  if(!t_mtnum
  || !ct_mtnum)
  {
    LogError("Miss trigger not found (T_MTNUM %i, CT_MTNUM %i)",
             t_mtnum, ct_mtnum);

    SetFailState(ERROR_MSG_FATAL);
    return;
  }

  UnhookEntityOutput(str_soccerball_class, str_soccerball_outp, Event_BallShot);
  UnhookEntityOutput("trigger_once", "OnTrigger", Event_GoalScored);
  UnhookEntityOutput("trigger_multiple", "OnStartTouch", Event_GoalMissed);

  HookEntityOutput(str_soccerball_class, str_soccerball_outp, Event_BallShot);
  HookEntityOutput("trigger_once", "OnTrigger", Event_GoalScored);
  HookEntityOutput("trigger_multiple", "OnStartTouch", Event_GoalMissed);

  // reference field size is defined as REFERENCE_FIELD_SIZE
  field_size = GetVectorDistance(vec_tgoal_origin, vec_ctgoal_origin);

  // calculate vectors between the goals
  SubtractVectors(vec_ctgoal_origin, vec_tgoal_origin, vec_tgoal_to_ctgoal);
  SubtractVectors(vec_tgoal_origin, vec_ctgoal_origin, vec_ctgoal_to_tgoal);

  // ForlixSoScore_OnEntitiesRead forward
  Call_StartForward(h_forward_entitiesread);
  Call_PushCell(soccerball_ref);
  Call_PushCell(goaltrig_t_ref);
  Call_PushCell(goaltrig_ct_ref);
  Call_PushArray(misstrig_t_refs, sizeof(misstrig_t_refs));
  Call_PushArray(misstrig_ct_refs, sizeof(misstrig_ct_refs));
  Call_PushFloat(field_size);
  Call_Finish();

  tracking_killed = false;
  return;
}


GetBallOrigin(Float:origin[3])
{
  GetEntPropVector(soccerball_ref, Prop_Data, "m_vecAbsOrigin", origin);
  return;
}


bool:IsInFrontOfGoal(const Float:vec_player_origin[3],
                     goal)
{
  decl Float:vec_goal_origin[3];
  decl Float:vec_goal_to_enemy_goal[3];

  switch(goal)
  {
    case CS_TEAM_T:
    {
      vec_goal_origin = vec_tgoal_origin;
      vec_goal_to_enemy_goal = vec_tgoal_to_ctgoal;
    }

    case CS_TEAM_CT:
    {
      vec_goal_origin = vec_ctgoal_origin;
      vec_goal_to_enemy_goal = vec_ctgoal_to_tgoal;
    }

    default:
      return(false);
  }

  if(GetVectorDistance(vec_player_origin, vec_goal_origin, true)
  >= Square(IN_FRONT_OF_GOAL_RADIUS/REFERENCE_FIELD_SIZE*field_size))
  // player is not in range of goal
    return(false);

  // get vector from goal to player
  decl Float:vec_goal_to_player[3];
  SubtractVectors(vec_player_origin, vec_goal_origin, vec_goal_to_player);

  if(GetVectorDotProduct(vec_goal_to_enemy_goal, vec_goal_to_player) <= 0)
  // player is behind the goal
    return(false);

  return(true);
}


KillTriggers()
{
  // doing disable here isnt enough as other entities might reenable them

  // kill goal triggers
  if(IsValidEntity(goaltrig_t_ref))
    AcceptEntityInput(goaltrig_t_ref, "Kill", -1, -1, 0);

  if(IsValidEntity(goaltrig_ct_ref))
    AcceptEntityInput(goaltrig_ct_ref, "Kill", -1, -1, 0);

  // kill miss triggers
  for(new i = 0; misstrig_t_refs[i] != INVALID_ENT_REFERENCE; i++)
  if(IsValidEntity(misstrig_t_refs[i]))
    AcceptEntityInput(misstrig_t_refs[i], "Kill", -1, -1, 0);

  for(new i = 0; misstrig_ct_refs[i] != INVALID_ENT_REFERENCE; i++)
  if(IsValidEntity(misstrig_ct_refs[i]))
    AcceptEntityInput(misstrig_ct_refs[i], "Kill", -1, -1, 0);

  tracking_killed = true;
  return;
}


FindEntityByTargetname(startindex,
                       const String:targetname[],
                       const String:classname[])
{
  decl String:namebuf[32];
  new index = startindex;
  namebuf[0] = '\0';

  while(strcmp(namebuf, targetname) != 0
    && (index = FindEntityByClassname(index, classname)) != -1)
    GetEntPropString(index, Prop_Data, "m_iName", namebuf, sizeof(namebuf));

  return(index);
}
