// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs


ResetTracking()
{
  // reset tracking
  shooter_userid = 0;
  lastshooter_userid = 0;

  shooter_team = 0;
  lastshooter_team = 0;

  ball_shottime = GetTickedTime();
  GetBallOrigin(ball_shotorigin);

  air_shot = false;
  return;
}


public Event_BallShot(const String:output[],
                      caller,
                      activator,
                      Float:delay)
{
  new caller_ref = EntIndexToEntRef(caller);

  if(caller_ref != soccerball_ref
  || roundend
  || round_abouttoend
  || tracking_killed)
    return;

  decl Float:ball_lastshottime;
  decl Float:ball_lastshotorigin[3];

  ball_lastshottime = ball_shottime;
  ball_lastshotorigin = ball_shotorigin;

  ball_shottime = GetTickedTime();
  GetBallOrigin(ball_shotorigin);

  new Float:distance = GetVectorDistance(ball_lastshotorigin, ball_shotorigin);
  new Float:distance_normalized = distance/field_size;
  new Float:speed_averaged = SafeFloatDiv(distance, ball_shottime-ball_lastshottime);

  lastshooter_userid = shooter_userid;
  lastshooter_team = shooter_team;

  if(activator > 0
  && activator <= MaxClients
  && IsClientInGame(activator))
  {
    shooter_userid = GetClientUserId(activator);
    shooter_team = GetClientTeam(activator);
  }
  else
  // non-player or world entity
  {
    shooter_userid = 0;
    shooter_team = 0;
  }

  // ForlixSoScore_OnBallShot forward
  Call_StartForward(h_forward_ballshot);
  Call_PushCell(shooter_userid);
  Call_PushCell(shooter_team);
  Call_Finish();

  if(shooter_userid != lastshooter_userid
  && shooter_userid != 0
  && lastshooter_userid != 0)
  // ball changed player and both are defined
  // lastshooter may no longer be ingame but shooter is
  {
    decl Float:shooter_noworigin[3];
    GetClientAbsOrigin(activator, shooter_noworigin);

    if(IsInFrontOfGoal(shooter_noworigin, shooter_team))
    // ball shot from in front of shooters teams goal
    {
      decl Handle:pack;
      CreateDataTimer(0.5, DelayedSaveTrack, pack, TIMER_DATA_HNDL_CLOSE);
      WritePackCell(pack, shooter_userid);
      WritePackCell(pack, shooter_team);
      WritePackCell(pack, lastshooter_userid);
      WritePackCell(pack, lastshooter_team);
      WritePackFloat(pack, distance_normalized);
      WritePackFloat(pack, speed_averaged);
      WritePackCell(pack, air_shot);
    }

    if(shooter_team == lastshooter_team)
    // friendly pass
    {
      if(distance_normalized
      >= MIN_PASS_DISTANCE/REFERENCE_FIELD_SIZE)
      {
        // ForlixSoScore_OnBallPassed forward
        Call_StartForward(h_forward_ballpassed);
        Call_PushCell(lastshooter_userid);
        Call_PushCell(shooter_userid);
        Call_PushCell(shooter_team);
        Call_PushFloat(distance_normalized);
        Call_PushFloat(speed_averaged);
        Call_PushCell(air_shot);
        Call_Finish();
      }
    }
    else
    // turnover
    {
      // ForlixSoScore_OnBallTurnedOver forward
      Call_StartForward(h_forward_ballturnedover);
      Call_PushCell(shooter_userid);
      Call_PushCell(shooter_team);
      Call_PushCell(lastshooter_userid);
      Call_PushCell(lastshooter_team);
      Call_PushFloat(distance_normalized);
      Call_PushFloat(speed_averaged);
      Call_PushCell(air_shot);
      Call_Finish();
    }
  }

  air_shot = false;
  return;
}


public Event_GoalScored(const String:output[],
                        caller,
                        activator,
                        Float:delay)
{
  new caller_ref = EntIndexToEntRef(caller);

  if(roundend
  || round_abouttoend
  || tracking_killed)
    return;

  decl losing_team;
  decl winning_team;

  // determine which goal was shot, or return if none
  if(caller_ref == goaltrig_t_ref)
  {
    losing_team = CS_TEAM_T;
    winning_team = CS_TEAM_CT;
  }
  else
  if(caller_ref == goaltrig_ct_ref)
  {
    losing_team = CS_TEAM_CT;
    winning_team = CS_TEAM_T;
  }
  else
    return;

  decl keeper_userids[MAX_KEEPER_NUM+1];
  new l = 0;

  // turn on takedamage for the lost teams players
  // and find out if there were keepers
  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client)
  && GetClientTeam(client) == losing_team)
  {
    p_died_from_goal[client] = true;

    SetEntityMoveType(client, MOVETYPE_WALK);
    SetEntProp(client, Prop_Data, "m_takedamage", 2, 1);

    decl Float:player_origin[3];
    GetClientAbsOrigin(client, player_origin);

    if(IsInFrontOfGoal(player_origin, losing_team)
    && l < MAX_KEEPER_NUM)
      keeper_userids[l++] = GetClientUserId(client);
  }

  keeper_userids[l] = 0;
  KillTriggers();

  new score_client = GetClientOfUserId(shooter_userid);
  new assist_client = GetClientOfUserId(lastshooter_userid);
  new losing_team_numplayers = GetTeamClientCount(losing_team);
  new winning_team_numplayers = GetTeamClientCount(winning_team);

  new shooter_userid_forward = 0;
  new assistant_userid_forward = 0;
  new bool:owngoal_forward = false;

  new bool:ban_score_client = false;

  if(losing_team_numplayers)
    round_abouttoend = true;

  if(score_client
  && IsClientInGame(score_client)
  && shooter_team == GetClientTeam(score_client))
  {
    decl String:steamid[32];
    decl String:scoreteam_name[32];

    GetTeamName(shooter_team, scoreteam_name, sizeof(scoreteam_name));

    if(shooter_team == winning_team)
    // normal goal
    {
      if(assist_client
      && IsClientInGame(assist_client)
      && assist_client != score_client
      && lastshooter_team == shooter_team
      && lastshooter_team == GetClientTeam(assist_client))
      {
        SayText2_NameBroadcast("CHAT_MSG_ASSIST", assist_client);

        SetDeaths(assist_client, ++p_assist[assist_client]);
        assistant_userid_forward = lastshooter_userid;

        if(log_goal_scores)
        {
          if(!GetClientAuthString(assist_client, steamid, sizeof(steamid)))
            strcopy(steamid, sizeof(steamid), "STEAM_ID_PENDING");

          LogToGame(LOG_MSG_ASSIST, assist_client, lastshooter_userid, steamid, scoreteam_name);
        }
      }

      SayText2_NameBroadcast("CHAT_MSG_GOAL", score_client);
      centermsg_type = "CENTER_MSG_GOAL";

      SetFrags(score_client, ++p_score[score_client], true);
      shooter_userid_forward = shooter_userid;

      if(log_goal_scores)
      {
        if(!GetClientAuthString(score_client, steamid, sizeof(steamid)))
          strcopy(steamid, sizeof(steamid), "STEAM_ID_PENDING");

        LogToGame(LOG_MSG_GOAL, score_client, shooter_userid, steamid, scoreteam_name);
      }
    }
    else
    // own goal
    {
      SayText2_NameBroadcast("CHAT_MSG_OWNGOAL", score_client);
      centermsg_type = "CENTER_MSG_OWNGOAL";

      if(own_goal_ban_num
      && winning_team_numplayers > 0
      && ++p_owngoal[score_client] >= own_goal_ban_num)
        ban_score_client = true;

      shooter_userid_forward = shooter_userid;
      owngoal_forward = true;

      if(log_goal_scores)
      {
        if(!GetClientAuthString(score_client, steamid, sizeof(steamid)))
          strcopy(steamid, sizeof(steamid), "STEAM_ID_PENDING");

        LogToGame(LOG_MSG_OWNGOAL, score_client, shooter_userid, steamid, scoreteam_name);
      }
    }

    if(!losing_team_numplayers)
    // goal scored but no one died, so round_end event won't be called
    {
      CenterMsg_NameBroadcast(centermsg_type, shooter_userid);
      printctmsg = false;
    }
    else
      printctmsg = true;
  }

  decl Float:ball_noworigin[3];
  GetBallOrigin(ball_noworigin);

  new Float:time_c = GetTickedTime();
  new Float:distance = GetVectorDistance(ball_shotorigin, ball_noworigin);
  new Float:time_into_round = time_c-round_freeze_end;

  // ForlixSoScore_OnGoalScored forward
  Call_StartForward(h_forward_goalscored);
  Call_PushCell(winning_team);
  Call_PushCell(losing_team);
  Call_PushCell(shooter_userid_forward);
  Call_PushCell(assistant_userid_forward);
  Call_PushArray(keeper_userids, sizeof(keeper_userids));
  Call_PushCell(owngoal_forward);
  Call_PushFloat(distance/field_size);
  Call_PushFloat(SafeFloatDiv(distance, time_c-ball_shottime));
  Call_PushCell(air_shot);
  Call_PushFloat(time_into_round);
  Call_Finish();

  if(ban_score_client)
  // if goal shooter needs to be banned, initiate timer
  {
    decl Handle:pack;
    decl String:steamid[32];

    if(GetClientAuthString(score_client, steamid, sizeof(steamid)))
    {
      CreateDataTimer(3.0, OwnGoalBanSteamID, pack, TIMER_DATA_HNDL_CLOSE);
      WritePackCell(pack, shooter_userid);
      WritePackString(pack, steamid);
    }
  }

  CreateTimer(0.25, TeamDeathFailSafe, losing_team);
  return;
}


public Action:TeamDeathFailSafe(Handle:timer,
                                any:team)
{
  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client)
  && GetClientTeam(client) == team)
  // make sure all losingteam players die
  // in case they're not affected by the trigger_hurt
    ForcePlayerSuicide(client);

  return(Plugin_Stop);
}


public Action:OwnGoalBanSteamID(Handle:timer,
                                Handle:pack)
{
  decl String:steamid[32];
  steamid[0] = '\0';

  ResetPack(pack);
  new client = GetClientOfUserId(ReadPackCell(pack));
  ReadPackString(pack, steamid, sizeof(steamid));

  if(!steamid[0])
    return(Plugin_Stop);

  decl String:reason[MAX_MSG_LEN];

  if(own_goal_ban_time)
  {
    decl String:ban_time[32];
    FriendlyTime(own_goal_ban_time*60, ban_time, sizeof(ban_time), false);

    Format(reason, sizeof(reason), OWN_GOAL_BAN_REASON, ban_time, own_goal_ban_num);
  }
  else
    Format(reason, sizeof(reason), OWN_GOAL_BAN_REASON_PERM, own_goal_ban_num);

  BanIdentity(steamid, own_goal_ban_time, BANFLAG_AUTHID, reason, "ForlixSoScore_OwnGoalBan");

  if(client)
    KickClient(client, "%s", reason);

  return(Plugin_Stop);
}


public Event_GoalMissed(const String:output[],
                        caller,
                        activator,
                        Float:delay)
{
  new caller_ref = EntIndexToEntRef(caller);

  if(roundend
  || round_abouttoend
  || tracking_killed)
    return;

  static Float:ball_lastshottime = 0.0;
  new missed_goal = 0;

  // see if a miss trigger called us
  // and find out the goal it belongs to

  // check against all T-team triggers
  for(new i = 0; misstrig_t_refs[i] != INVALID_ENT_REFERENCE; i++)
  if(caller_ref == misstrig_t_refs[i])
  {
    missed_goal = CS_TEAM_T;
    break;
  }

  if(!missed_goal)
  // if not found yet
  // check against all CT-team triggers
  for(new i = 0; misstrig_ct_refs[i] != INVALID_ENT_REFERENCE; i++)
  if(caller_ref == misstrig_ct_refs[i])
  {
    missed_goal = CS_TEAM_CT;
    break;
  }

  if(!missed_goal
  || ball_shottime-ball_lastshottime < MIN_MISS_INTERVAL)
  // not called by a miss trigger entity
  // or ball has not been shot since last call
    return;

  decl Float:ball_noworigin[3];
  GetBallOrigin(ball_noworigin);

  new Float:distance = GetVectorDistance(ball_shotorigin, ball_noworigin);

  decl Handle:pack;
  CreateDataTimer(0.5, DelayedMissTrack, pack, TIMER_DATA_HNDL_CLOSE);
  WritePackCell(pack, shooter_userid);
  WritePackCell(pack, shooter_team);
  WritePackCell(pack, missed_goal);
  WritePackFloat(pack, distance/field_size);
  WritePackFloat(pack, SafeFloatDiv(distance, GetTickedTime()-ball_shottime));
  WritePackCell(pack, air_shot);

  ball_lastshottime = ball_shottime;
  return;
}


public Action:DelayedMissTrack(Handle:timer, Handle:pack)
{
  if(roundend
  || round_abouttoend
  || tracking_killed)
    return(Plugin_Stop);

  ResetPack(pack);

  // ForlixSoScore_OnGoalMissed forward
  Call_StartForward(h_forward_goalmissed);
  Call_PushCell(ReadPackCell(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_PushFloat(ReadPackFloat(pack));
  Call_PushFloat(ReadPackFloat(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_Finish();

  return(Plugin_Stop);
}


public Action:DelayedSaveTrack(Handle:timer, Handle:pack)
{
  if(roundend
  || round_abouttoend
  || tracking_killed)
    return(Plugin_Stop);

  decl Float:ball_noworigin[3];
  GetBallOrigin(ball_noworigin);

  new Float:distance_normalized = GetVectorDistance(ball_shotorigin, ball_noworigin)/field_size;

  if(distance_normalized
  < MIN_SAVE_DISTANCE/REFERENCE_FIELD_SIZE)
  // ball was not shot far enough away for it to be a save
    return(Plugin_Stop);

  ResetPack(pack);

  // ForlixSoScore_OnGoalSaved forward
  Call_StartForward(h_forward_goalsaved);
  Call_PushCell(ReadPackCell(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_PushFloat(ReadPackFloat(pack));
  Call_PushFloat(ReadPackFloat(pack));
  Call_PushCell(ReadPackCell(pack));
  Call_Finish();

  return(Plugin_Stop);
}
