// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs


ResetPlayerHealth(bool:normal=false)
{
  new takedmg = 2;
  new health = 100;

  if(!normal
  && player_godmode_health)
  {
    takedmg = 0;
    health = player_godmode_health;
  }

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client))
  {
    SetEntProp(client, Prop_Data, "m_takedamage", takedmg, 1);
    SetEntProp(client, Prop_Data, "m_iHealth", health, 1);
  }

  return;
}


LateLoad()
{
  // hot load or reload of plugin
  // sync scores, spawn dead players, remove ragdolls
  round_freeze_end = GetTickedTime();
  SetupEntities();

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client))
  {
    p_score[client] = GetClientFrags(client);
    p_assist[client] = GetClientDeaths(client);

    p_owngoal[client] = 0;

    if(p_score[client] < 0)
    {
      p_score[client] = 0;
      SetFrags(client, 0, true);
    }

    if(p_assist[client] < 0)
    {
      p_assist[client] = 0;
      SetDeaths(client, 0);
    }

    if(!IsPlayerAlive(client))
    {
      RemoveRagdoll(client);
      RequestPlayerSpawn(client);
    }
  }

  return;
}


SetFrags(client, frags, bool:update)
{
  SetEntProp(client, Prop_Data, "m_iFrags", frags);

  if(update)
    AcceptEntityInput(game_score_ref, "ApplyScore", client, game_score_ref);

  return;
}


SetDeaths(client, deaths)
{
  SetEntProp(client, Prop_Data, "m_iDeaths", deaths);
  return;
}


PrintAnnouncementMsg(client)
{
  if(client)
  // print to single client
    PrintToChat(client, "%s %t", ANNOUNCEMENT_PREFIX, "ANNOUNCEMENT_MSG", help_cmd);
  else
  // broadcast to everyone
  for(new i = 1; i <= MaxClients; i++)
  if(IsClientInGame(i))
    PrintToChat(i, "%s %t", ANNOUNCEMENT_PREFIX, "ANNOUNCEMENT_MSG", help_cmd);

  return;
}


SayText2_NameBroadcast(const String:mtype[],
                       client)
{
  decl String:name[MAX_NAME_LENGTH];
  if(!GetClientName(client, name, sizeof(name)))
    return;

  for(new i = 1; i <= MaxClients; i++)
  if(IsClientInGame(i))
  {
    decl String:msg[MAX_MSG_LEN];
    Format(msg, sizeof(msg), "%T", mtype, i, name);

    new Handle:h_saytext2 = StartMessageOne("SayText2", i,
    USERMSG_RELIABLE|USERMSG_BLOCKHOOKS);

    if(h_saytext2 != INVALID_HANDLE)
    {
      BfWriteByte(h_saytext2, client);
      BfWriteByte(h_saytext2, true);
      BfWriteString(h_saytext2, msg);
      EndMessage();
    }
  }

  return;
}


CenterMsg_NameBroadcast(const String:mtype[],
                        userid)
{
  new client = GetClientOfUserId(userid);
  decl String:name[MAX_NAME_LENGTH];
  decl String:name_esc[MAX_NAME_LENGTH*2];

  if(!client
  || !IsClientInGame(client)
  || !GetClientName(client, name, sizeof(name)))
    return;

  new r = 0;
  new w = 0;

  // escape all ampersands for centermsg display
  if(name[0])
  do
  {
    if(name[r] == '&')
    {
      name_esc[w++] = '&';
      name_esc[w++] = '&';
    }
    else
      name_esc[w++] = name[r];
  }
  while(name[++r]);
  name_esc[w] = '\0';

  for(new i = 1; i <= MaxClients; i++)
  if(IsClientInGame(i))
    PrintCenterText(i, "%T", mtype, i, name_esc);

  return;
}


bool:DeadTimeBlock(client)
{
  // block all calls within defined deadtime
  static Float:time_nc[MAXPLAYERS+1];
  new Float:time_c = GetTickedTime();

  if(time_c < time_nc[client])
    return(true);

  time_nc[client] = time_c + PLUGIN_COMMAND_DEADTIME;
  return(false);
}


Float:SafeFloatDiv(Float:dividend, Float:divisor)
{
  if(!divisor)
    return(0.0);
  else
    return(dividend/divisor);
}


bool:FriendlyTime(time_s,
                  String:str_ftime[],
                  str_ftime_len,
                  bool:compact=true)
{
  decl String:days_pf[16];
  decl String:hrs_pf[16];
  decl String:mins_pf[16];
  decl String:secs_pf[16];

  if(compact)
  {
    days_pf = "d";
    hrs_pf = "h";
    mins_pf = "m";
    secs_pf = "s";
  }
  else
  {
    days_pf = " days";
    hrs_pf = " hours";
    mins_pf = " minutes";
    secs_pf = " seconds";
  }

  if(time_s < 0)
  {
    str_ftime[0] = '\0';
    return(false);
  }

  new days = time_s/86400;
  new hrs = (time_s/3600)%24;
  new mins = (time_s/60)%60;
  new secs = time_s%60;

  if(time_s < 60)
    Format(str_ftime, str_ftime_len,
           "%u%s",
           secs, secs_pf);
  else
  if(time_s < 3600)
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           mins, mins_pf, secs, secs_pf);
  else
  if(time_s < 86400)
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           hrs, hrs_pf, mins, mins_pf);
  else
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           days, days_pf, hrs, hrs_pf);

  return(true);
}


Float:Square(Float:float_to_square)
{
  return(float_to_square*float_to_square);
}


stock FlashDamage(client, Float:origin[3])
{
  new Handle:h_damage = StartMessageOne("Damage",
  client,
  USERMSG_RELIABLE|USERMSG_BLOCKHOOKS);

  if(h_damage != INVALID_HANDLE)
  {
    BfWriteByte(h_damage, 1);
    BfWriteVecCoord(h_damage, origin);
    EndMessage();
  }

  return;
}
