#include <sourcemod>
#include <sdktools>

#pragma semicolon 1

#define PLUGIN_VERSION            "1.4"

new bool:p_debugto[MAXPLAYERS+1];

public Plugin:myinfo =
{
  name = "ForlixSoScore ForwardSample",
  author = "Forlix (Dominik Friedrichs)",
  description = "ForlixSoScore ForwardSample",
  version = PLUGIN_VERSION,
  url = "http://forlix.org/"
};


public OnPluginStart()
{
  RegConsoleCmd("fs_debug", DebugMsgToggle);
  return;
}


public OnClientConnected(client)
{
  p_debugto[client] = false;
  return;
}


public ForlixSoScore_OnNoSoccerMap()
{
  // called right after OnMapStart
  PrintToServer("SoScore reports this is not a known soccer map");
  return;
}


public ForlixSoScore_OnEntitiesRead(soccerball_ref,
                                    goaltrig_t_ref,
                                    goaltrig_ct_ref,
                                    const misstrig_t_refs[],
                                    const misstrig_ct_refs[],
                                    Float:field_size)
{
  // called right after OnMapStart on late loads
  // and right after each Round_Start
  decl String:str_buf[256];
  decl String:str_targetname[256];

  GetEntPropString(soccerball_ref, Prop_Data, "m_iName", str_targetname, sizeof(str_targetname));
  Format(str_buf, sizeof(str_buf), "soccerball_ref: %i (%s)", soccerball_ref, str_targetname);
  DebugMsg(str_buf);

  GetEntPropString(goaltrig_t_ref, Prop_Data, "m_iName", str_targetname, sizeof(str_targetname));
  Format(str_buf, sizeof(str_buf), "goaltrig_t_ref: %i (%s)", goaltrig_t_ref, str_targetname);
  DebugMsg(str_buf);

  GetEntPropString(goaltrig_ct_ref, Prop_Data, "m_iName", str_targetname, sizeof(str_targetname));
  Format(str_buf, sizeof(str_buf), "goaltrig_ct_ref: %i (%s)", goaltrig_ct_ref, str_targetname);
  DebugMsg(str_buf);

  Format(str_buf, sizeof(str_buf), "misstrig_t_refs: ");

  for(new i = 0; misstrig_t_refs[i] != INVALID_ENT_REFERENCE; i++)
  {
    decl String:a[32];
    decl String:b[64];

    GetEntPropString(misstrig_t_refs[i], Prop_Data, "m_iName", a, sizeof(a));
    Format(b, sizeof(b), "%i (%s), ", misstrig_t_refs[i], a);

    StrCat(str_buf, sizeof(str_buf), b);
  }

  DebugMsg(str_buf);

  Format(str_buf, sizeof(str_buf), "misstrig_ct_refs: ");

  for(new i = 0; misstrig_ct_refs[i] != INVALID_ENT_REFERENCE; i++)
  {
    decl String:a[32];
    decl String:b[64];

    GetEntPropString(misstrig_ct_refs[i], Prop_Data, "m_iName", a, sizeof(a));
    Format(b, sizeof(b), "%i (%s), ", misstrig_ct_refs[i], a);

    StrCat(str_buf, sizeof(str_buf), b);
  }

  DebugMsg(str_buf);

  Format(str_buf, sizeof(str_buf), "field_size: %.2f", field_size);
  DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnBallShot(shooter_userid,
                                shooter_team)
{
  decl String:str_buf[256];

  decl String:str_shooter_name[32];
  decl String:str_shooter_team[32];

  new shooter = GetClientOfUserId(shooter_userid);

  if(!shooter
  || !IsClientInGame(shooter)
  || !GetClientName(shooter, str_shooter_name, sizeof(str_shooter_name)))
  {
    str_shooter_name = "N/A";
    str_shooter_team = "N/A";
  }
  else
    GetTeamName(shooter_team, str_shooter_team, sizeof(str_shooter_team));

  Format(str_buf, sizeof(str_buf), "\x03%s\x01 (%s) shot",
  str_shooter_name, str_shooter_team);

  //DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnBallPassed(passedfrom_userid,
                                  passedto_userid,
                                  team,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot)
{
  decl String:str_buf[256];

  decl String:str_from_name[32];
  decl String:str_to_name[32];

  decl String:str_team[32];

  new from = GetClientOfUserId(passedfrom_userid);
  new to = GetClientOfUserId(passedto_userid);

  if(!from
  || !IsClientInGame(from)
  || !GetClientName(from, str_from_name, sizeof(str_from_name)))
    str_from_name = "N/A";

  if(!to
  || !IsClientInGame(to)
  || !GetClientName(to, str_to_name, sizeof(str_to_name)))
    str_to_name = "N/A";

  GetTeamName(team, str_team, sizeof(str_team));

  Format(str_buf, sizeof(str_buf), "\x03%s\x01 passed \x03%s\x01 (%s), d %.2f, s %.2f, air %u",
  str_from_name, str_to_name, str_team, distance_normalized, speed_averaged, air_shot);

  DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnBallTurnedOver(stealer_userid,
                                      stealer_team,
                                      loser_userid,
                                      loser_team,
                                      Float:distance_normalized,
                                      Float:speed_averaged,
                                      bool:air_shot)
{
  decl String:str_buf[256];

  decl String:str_stealer_name[32];
  decl String:str_loser_name[32];

  decl String:str_stealer_team[32];
  decl String:str_loser_team[32];

  new stealer = GetClientOfUserId(stealer_userid);
  new loser = GetClientOfUserId(loser_userid);

  if(!stealer
  || !IsClientInGame(stealer)
  || !GetClientName(stealer, str_stealer_name, sizeof(str_stealer_name)))
  {
    str_stealer_name = "N/A";
    str_stealer_team = "N/A";
  }
  else
    GetTeamName(stealer_team, str_stealer_team, sizeof(str_stealer_team));

  if(!loser
  || !IsClientInGame(loser)
  || !GetClientName(loser, str_loser_name, sizeof(str_loser_name)))
  {
    str_loser_name = "N/A";
    str_loser_team = "N/A";
  }
  else
    GetTeamName(loser_team, str_loser_team, sizeof(str_loser_team));

  Format(str_buf, sizeof(str_buf), "\x03%s\x01 (%s) stole \x03%s\x01 (%s), d %.2f, s %.2f, air %u",
  str_stealer_name, str_stealer_team,
  str_loser_name, str_loser_team,
  distance_normalized, speed_averaged, air_shot);

  DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnGoalScored(winning_team,
                                  losing_team,
                                  shooter_userid,
                                  assistant_userid,
                                  keeper_userids[],
                                  bool:owngoal,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot,
                                  Float:time_into_round)
{
  decl String:str_buf[256];

  decl String:str_winning_team[32];
  decl String:str_losing_team[32];

  decl String:str_shooter_name[32];
  decl String:str_assistant_name[32];

  // teams are guaranteed to be provided, no need to check
  GetTeamName(winning_team, str_winning_team, sizeof(str_winning_team));
  GetTeamName(losing_team, str_losing_team, sizeof(str_losing_team));

  new shooter = GetClientOfUserId(shooter_userid);
  new assistant = GetClientOfUserId(assistant_userid);

  if(!shooter
  || !IsClientInGame(shooter)
  || !GetClientName(shooter, str_shooter_name, sizeof(str_shooter_name)))
    str_shooter_name = "N/A";

  if(!assistant
  || !IsClientInGame(assistant)
  || !GetClientName(assistant, str_assistant_name, sizeof(str_assistant_name)))
    str_assistant_name = "N/A";

  Format(str_buf, sizeof(str_buf),
  "\x01Team %s won, team %s lost!\nShooter: \x03%s\x01, Assistant: \x03%s\x01\nOwngoal: %s",
  str_winning_team, str_losing_team, str_shooter_name, str_assistant_name, owngoal? "yes" : "no");

  DebugMsg(str_buf);

  Format(str_buf, sizeof(str_buf), "\x01Keepers: \x03");

  // keeper list is zero-terminated, dynamic length
  for(new i = 0; keeper_userids[i] > 0; i++)
  {
    decl String:str_keeper_name[32];

    new keeper = GetClientOfUserId(keeper_userids[i]);

    if(!keeper
    || !IsClientInGame(keeper)
    || !GetClientName(keeper, str_keeper_name, sizeof(str_keeper_name)))
      str_keeper_name = "N/A";

    StrCat(str_buf, sizeof(str_buf), str_keeper_name);
    StrCat(str_buf, sizeof(str_buf), "\x01, \x03");
  }

  DebugMsg(str_buf);

  Format(str_buf, sizeof(str_buf), "Dist: %.2f AvgSpeed: %.2f, Air: %u, Time into round: %.2fs",
  distance_normalized, speed_averaged, air_shot, time_into_round);

  DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnGoalMissed(shooter_userid,
                                  shooter_team,
                                  missed_goal,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot)
{
  decl String:str_buf[256];

  decl String:str_shooter_name[32];
  decl String:str_shooter_team[32];
  decl String:str_missed_goal[32];

  new shooter = GetClientOfUserId(shooter_userid);

  if(!shooter
  || !IsClientInGame(shooter)
  || !GetClientName(shooter, str_shooter_name, sizeof(str_shooter_name)))
    str_shooter_name = "N/A";
  else
    GetTeamName(shooter_team, str_shooter_team, sizeof(str_shooter_team));

  GetTeamName(missed_goal, str_missed_goal, sizeof(str_missed_goal));

  Format(str_buf, sizeof(str_buf), "\x03%s\x01 (%s) missed \x03%s\x01 goal, d %.2f, s %.2f, air %u",
  str_shooter_name, str_shooter_team, str_missed_goal,
  distance_normalized, speed_averaged, air_shot);

  DebugMsg(str_buf);

  return;
}


public ForlixSoScore_OnGoalSaved(keeper_userid,
                                 keeper_team,
                                 shooter_userid,
                                 shooter_team,
                                 Float:distance_normalized,
                                 Float:speed_averaged,
                                 bool:air_shot)
{
  decl String:str_buf[256];

  decl String:str_keeper_name[32];
  decl String:str_keeper_team[32];

  decl String:str_shooter_name[32];
  decl String:str_shooter_team[32];

  new keeper = GetClientOfUserId(keeper_userid);
  new shooter = GetClientOfUserId(shooter_userid);

  if(!keeper
  || !IsClientInGame(keeper)
  || !GetClientName(keeper, str_keeper_name, sizeof(str_keeper_name)))
    str_keeper_name = "N/A";
  else
    GetTeamName(keeper_team, str_keeper_team, sizeof(str_keeper_team));

  if(!shooter
  || !IsClientInGame(shooter)
  || !GetClientName(shooter, str_shooter_name, sizeof(str_shooter_name)))
    str_shooter_name = "N/A";
  else
    GetTeamName(shooter_team, str_shooter_team, sizeof(str_shooter_team));

  Format(str_buf, sizeof(str_buf), "\x03%s\x01 (%s) saved from \x03%s\x01 (%s), d %.2f, s %.2f, air %u",
  str_keeper_name, str_keeper_team,
  str_shooter_name, str_shooter_team,
  distance_normalized, speed_averaged, air_shot);

  DebugMsg(str_buf);

  return;
}


public Action:DebugMsgToggle(client, args)
{
  if(!client)
    return(Plugin_Continue);

  if(!p_debugto[client])
  {
    p_debugto[client] = true;
    PrintToChat(client, "Debug messages enabled.");
  }
  else
  {
    p_debugto[client] = false;
    PrintToChat(client, "Debug messages disabled.");
  }

  return(Plugin_Handled);
}


DebugMsg(const String:msg[])
{
  for(new i = 1; i <= MaxClients; i++)
  if(IsClientInGame(i)
  && p_debugto[i])
    PrintToChat(i, msg);

  return;
}
